%--------------------------------------------------------------------%
%----------------------BLOWOUT SIMULATOR-----------------------------%
%--------------------------------------------------------------------%

%--------------------------------------------------------------------%
%Written by:  Kristoffer Evensen, Spring 2013                        %
%Description: This MATLAB probram will calculate the stabilized      %
%             blowout rate for the given well parameters             %
%             (specified in input the section)                       %
%             In addition to this, the program can simulate how      %
%             the influx rate will change if dynamic kill fluid is   %
%             injected into the well at different depths             %
%Purpose:     The program were prepared to simulate the benefits of  %
%             intersecting the blowing well at a deeper point than   %
%             currently possible with existing technology.           %
%             The program can calculate both the difference          %
%             in pumping rates, pressure and static kill fluid       %
%             required.                                              %
%--------------------------------------------------------------------%


%--------------------------------------------------------------------%
%----------------------- First Order Input Variables ----------------&
%--------------------------------------------------------------------%

%Dimensions            Description:                 Unit: 
TD=3000;               %Terminal depth of well      (meters)
CSD=2000;              %Casing shoe depth           (meters)
Pd=5;                  %Pipe diameter               (inch)
OHD=8.5;               %Open hole diameter          (inch)
CID=9.063;             %Casing inside diameter      (inch)
 
%Properties
RT=100;                %Reservoir temperature       (Celsius)
FFD=461;               %formation fluid density 
                       %@ in-situ conditions        (kg/m3)
RP=400;                %Reservoir pressure          (bar)
WD=1000;               %fresh water density         (kg/m3)                  
SP=15;                 %Wellhead pressure           (bar)
SWD=25;                %Dissolved salt              (kg/m3)
ST=4;                  %Wellhead temperature        (Celsius)

%Wellbore roughness
Cr=0.1;                %Casing absolute roughness   (mm)
Fr=1;                  %Formation absolute roughness(mm)

%Intersection Point 
ID=3000;               %Interception depth          (meter)

%Injection Rate
RR=0;                  %relief well in-situ rate    (l/s)

%--------------------------------------------------------------------%
%------------------- Second Order Input Variables -------------------&
%--------------------------------------------------------------------%
 
%                   Description:                              Unit:
HCRF=2500;          %Heat Capacity Of Reservoir Fluid         (J/kg-C)
HCI=4300;           %Heat Capacity Of Injection Water         (J/kg-C)
U=0.6;              %Thermal conductivity of cased section    (W/mK)
U2=2.37;            %Thermal conductivity of openhole section (W/mk)
hr=0.001;           %heat transfer across pipe            (btu/hrsqftF)
Mi=45.43;           %Initial molar mass of reservoir fluid  (kg/k-mol)                    
InvPoint=0.5;       %Inversion Point of immiscible liquids    (ratio)
R=0.1;              %Initial blowout rate (to be adjusted)    (l/s)

%--------------------------------------------------------------------%
%---------------------- Initial Calculations ------------------------&
%--------------------------------------------------------------------%
 
A=zeros(TD,16);     %Answer matrix
                    %(depth(m), depth(ft), wellbore pressure (bar), 
                    %flow diameter (m), frictional pressure drop (bar)
                    %hydrostatic pressure drop (bar), flow velocity
                    %(m/s), roughness (mm), liquid density (kg/m3)
                    %mixture density (kg/m3), flow regime
                    %gas density (kg/m3), two phase friction factor, 
                    %no-slip friction factor)

PF=Pd/12;           %Pipe diameter in feet      
G=(RT-ST)/TD;       %Geothermal gradient (C/m)
GF=G*1.8/3.28084;   %Geothermal gradient (F/ft)
TDF=TD*3.28084;     %Well depth (ft)
CDF=CSD*3.28084;    %Casing shoe depth (ft)
 
%--------------------------------------------------------------------%
%----------------------- Temperature profile ------------------------&
%--------------------------------------------------------------------%

Constraint=500;
ST2=ST*(9/5)+3;
CT2=ST2+GF*CDF;
mri=(980+SWD)*RR; 
mrr=(461.2)*R;

while Constraint>(SP)

if ID==3000
IT=(1.89688*10^-9*RR^4-2.87808*10^-6*RR^3+0.001710826*RR^2-0.514208719*RR+100.8124059);
else
IT=(4.11744*10^-9*RR^4-4.7389*10^-6*RR^3+0.002088315*RR^2-0.452446577*RR+67.92743941);
end

if ID==3000
RT3=IT*(mri/(mrr+mri))+RT*(mrr/(mri+mrr));
else 
RT3=RT;
end

RT2=RT3*(9/5)+32;

FFD2=(R/(R+RR))*FFD+(RR/(R+RR))*995;

m=(((R+RR)*FFD2/1000)/0.4536)*60*60; %lb/hr mass flow
U4=U*0.578/(CID/12); %Actual thermal conductivity btu/F sq ft hr cased interval
U3=U2*0.578/(OHD/12); %Open Hole

if ID==3000
   Cp=(HCRF*(mrr/(mrr+mri))+HCI*(mri/(mrr+mri)))*0.000238846; %Specific heat capacity of mud, assumed constant
   Cp2=(HCRF*(mrr/(mrr+mri))+HCI*(mri/(mrr+mri)))*0.000238846;
   
else
   Cp2=HCRF*0.000238846;
   Cp=(HCRF*(mrr/(mrr+mri))+HCI*(mri/(mrr+mri)))*0.000238846;
end  
   


TA1=m*Cp2/(2*pi*PF*hr);
TB1=(OHD/(2*12))*U3/((Pd/(2*12))*hr);
TC11=(TB1/(2*TA1))*(1+(1+(4/TB1))^0.5);
TC21=(TB1/(2*TA1))*(1-(1+(4/TB1))^0.5);
TC31=1+(TB1/2)*(1+(1+(4/TB1))^0.5);
TC41=1+(TB1/2)*(1-(1+(4/TB1))^0.5);
GA1=GF*TA1;
EC1H1=exp(TC11*(TDF-CDF));
EC2H1=exp(TC21*(TDF-CDF));
K21=(GA1-(RT2-CT2+GA1)*EC1H1*(1-TC31))/(EC2H1*(1-TC41)-EC1H1*(1-TC31));
K11=RT2-K21-CT2+GA1;

TA2=m*Cp/(2*pi*PF*hr);
TB2=(CID/(2*12))*U4/((Pd/(2*12))*hr);
TC12=(TB2/(2*TA2))*(1+(1+(4/TB2))^0.5);
TC22=(TB2/(2*TA2))*(1-(1+(4/TB2))^0.5);
TC32=1+(TB2/2)*(1+(1+(4/TB2))^0.5);
TC42=1+(TB2/2)*(1-(1+(4/TB2))^0.5);
GA2=GF*TA2;
EC1H2=exp(TC12*TDF);
EC2H2=exp(TC22*TDF);
K22=(GA2-(RT2-ST2+GA2)*EC1H2*(1-TC32))/(EC2H2*(1-TC42)-EC1H2*(1-TC32));
K12=RT2-K22-ST2+GA2;
testz=K11*TC31*exp(TC11*0)+K21*TC41*exp(TC21*(0))+GF*((0))+CT2;
testy=K12*TC32*exp(TC12*ID*3.28084)+K22*TC42*exp(TC22*ID*3.38084)+GF*ID*3.28084+ST2;
diff=testz-testy;

%--------------------------------------------------------------------%
%--------------------- Pressure Calculations ------------------------&
%--------------------------------------------------------------------%

%----------------------- Flowing Fractions --------------------------&

for i=1:3000
    id=3001-i;
    
    if id<CSD
        A(id,5)=CID*(2.54/100);
        A(id,9)=Cr;
    else
        A(id,5)=OHD*(2.54/100);
        A(id,9)=Fr;
    end
    
    ift=id*3.28084;
    A(id,1)=id;
    A(id,2)=ift;
    
    if ID==3000
        mixc=0; 
    else
        mixc1=((IT*1.8)+32)*(mri/(mri+mrr))+(K12*TC32*exp(TC12*ID*3.28084)+K22*TC42*exp(TC22*ID*3.28084)+GF*ID*3.28084+ST2+diff)*(mrr/(mrr+mri));
        mixc2=K12*TC32*exp(TC12*ID*3.28084)+K22*TC42*exp(TC22*ID*3.28084)+GF*ID*3.28084+ST2+diff;
        mixc=mixc2-mixc1;
    end
    
    if id>=CSD
    AF=K11*TC31*exp(TC11*(ift-(ID*3.28084)))+K21*TC41*exp(TC21*(ift-(ID*3.28084)))+GF*(ift-(ID*3.28084))+CT2;
    A(id,3)=(AF-32)*(5/9);
    else
    AF=K12*TC32*exp(TC12*ift)+K22*TC42*exp(TC22*ift)+GF*ift+ST2+diff-mixc;
    A(id,3)=(AF-32)*(5/9);  
    end
 
    G(id,1)=(AF-32)*(5/9);
end

for i=1:300
    id=i*10;
    Gtemp(i,1)=G(id,1);
end

A(3000,4)=RP;

%----------------------- Flowing Fractions --------------------------&

for i=0:2998
   
    depth=3000-i;
    p=A(depth,4);
    
    if p>290 
        OV2=1;  
    else
        OV2=4.53264*10^-12*p^5-3.0606*10^-9*p^4+7.8059*10^-7*p^3-8.98995*10^-5*p^2+0.006172453*p+0.074089141;
    end
    
%--------------------------- Molar Mass -----------------------------&
    if p>290
        OM=45.43510681;
        GM=0;
    else
        OM=-7.07369*10^-10*p^5+5.96024*10^-7*p^4-0.000190337*p^3+0.029068761*p^2-2.429140018*p+181.9311708;
        GM=1.89868*10^-12*p^6-1.80307*10^-9*p^5+6.67172*10^-7*p^4-0.000120939*p^3+0.011206502*p^2-0.497904762*p+32.49449801;
    end

%------------------------ Fluid properties --------------------------&
 
%Pseudo reduced properties from Standings correlation,
%Z-factor from Hall & Yarborogh

    T=A(depth,3)+273.15; %Temperature Kelvin
    P=A(depth,4); %Pressure Bar
    Pp=P*14.5037738; %Pressure Psia
    Tc=A(depth,3); %Temperature Celcius
    Tr=T*1.8;
    rough=A(depth,9);
    
    if P<290
       SG=GM/28.97;
    if SG<0.75
        Tpc=(168+325*SG-12.5*SG^2)*0.555555556;
        Ppc=(667+15*SG-37.5*SG^2)*(10^5/14.5);
    elseif SG>=0.75
        Tpc=(187+330*SG-71.5*SG^2)*0.555555556;
        Ppc=(706+51.7*SG-1.11*SG^2)*(10^5/14.5);
    end
    Tpr=T/Tpc;
    Ppr=P*10^5/Ppc;
    
    %Hall-Yarborogh
    
    t=1/Tpr;
    a=0.06125*t*exp(-1.2*(1-t)^2);
    y=0.01;                       %random value, changed through iteration
    ytest=1;
    
    %iteration to solve the Hall-Yarborogh equation
    %is performed by the Newton Raphso method. 

    while ytest>10^-5 %constrain can be increased to increase effectivity
       fy=-a*Ppr+(y+y^2+y^3-y^4)/(1-y)^3-(14.76*t-9.76*t^2+4.58*t^3)*y^2+(90.7*t-242.2*t^2+42.4*t^3)*y^(2.18+2.82*t);
       dfy=(1+4*y+4*y^2-4*y^3+y^4)/(1-y)^4-(29.52*t-19.52*t^2+9.16*t^3)*y+(2.18+2.82*t)*(90.7*t-242.2*t^2+42.4*t^3)*y^(1.18+2.82*t);
       y0=y;
       y=y-(fy/dfy);
       ytest=(y-y0)^2; 
    end
       z=a*Ppr/y;
    else 
        z=1;
    end
    
%--------------------------- Densities ------------------------------&
    
    GD=(z*P*10^5*GM)/(8.3144621*T*1000); %kg/m3
    GDa=GD*0.0624279606; %lb/ft3

    if p>290     
       OD=0.330750716*p+345.4400112;
       viscoil=4.47062*10^-5*p+0.06627409;
       ODi=0.330750716*RP+345.4400112;
       ST=2.790286092;
    else
       OD=6.50336*10^-12*p^6-6.98355*10^-9*p^5+2.8333*10^-6*p^4-0.000543422*p^3+0.051180284*p^2-3.103346739*p+707.0146945;
       viscoil=-0.129609989*log(p)+0.833227262;
       ODi=0.330750716*RP+345.4400112;
       ST=-1.02387*10^-10*p^5+8.46729*10^-8*p^4-2.62993*10^-5*p^3+0.003854784*p^2-0.301653023*p+18.49224691;
    end

%-------------------------- Molar Flow ------------------------------%
    
    Q=(R/1000)*ODi/(87.4/1000);
   
    if depth>ID
        WD=0.033746509*p+(980.4005001+SWD);
        viscwater=1.36027*10^-8*p+0.463103407;
        WDi=0.033746509*A(ID,4)+(980.4005001+SWD);
        Qw=0;
        
    else 
        WD=0.033746509*p+(980.4005001+SWD);
        viscwater=1.36027*10^-8*p+0.463103407;
        WDi=0.033746509*A(ID,4)+(980.4005001+SWD);
        Qw=(RR/1000)*WDi/(18.02/1000);
    end

    OF=Q*OV2;
    OG=Q*(1-OV2);
    OW=Qw;
    MA=OM*OV2+GM*(1-OV2);
    
%--------------------- Volumetric flow ------------------------------%
   
    if GD==0
       GVF=0;
    else
        GVF=OG*GM/(1000*GD); %cubic metre per second
    end
    
    OVF=OF*(OM/1000)/OD; %cubic metre per second
    WVF=OW*(18.02/1000)/WD;
    MVF=GVF+OVF+WVF;
    LVF=WVF+OVF;
    LV=LVF/MVF;
    
%----------------------- Flow Velocity ------------------------------%
    
    D=A(depth,5);
    Area=pi*(D/2)^2;
    V=(OVF+GVF+WVF)/Area;
    A(depth,8)=V;
     
 %-------------------------- Viscosity -------------------------------%
        
    ratio=(WVF/LVF);
    
    if ratio <= InvPoint
       viscliquid=viscoil*(1/(1-ratio)^2.5); 
    else 
       viscliquid=viscwater*(1/ratio^2.5);
    end
   
    if p<290
    
       Rg=(0.001031*Pp*Tr)/(Pp-(0.061*Tr));
       Bg=(0.0283*z*Tr)/Pp;
       xg=Rg/(100*Bg);
       viscgas=1.6*(0.0109388-0.0088234*xg-0.0075720*xg^2)/(1.0-1.3633077*xg+0.0461989*xg^2); 
       %In situ gas viscosity (cp)
    else
        viscgas=0;
    end
    
    mixturevisc=viscliquid*LVF/MVF+viscgas*GVF/MVF;
    
%-------------- Pressure Loss - Beggs and Brill ---------------------%
    
    gft=32.2;
    D=A(depth, 5)*3.28084;
    um=MVF*35.3147;
    ul=LVF*35.3147;
    Cl=ul/um;
    Nfr=um^2/(gft*D);
    L1=316*Cl^0.302;
    L2=0.0009252*Cl^-2.4684;
    L3=0.10*Cl^-1.4516;
    L4=0.5*Cl^-6.738;
    ulum=ul-um;
    Vsl=(LVF/Area)*3.28084; %Superficial liquuid velocity
    liquiddensity=(WVF/LVF)*WD+(OVF/LVF)*OD;
    liquiddensitya=liquiddensity*0.0624279606;
    Nvl=1.938*Vsl*(liquiddensitya/(gft*ST));
    beta=0;
    Elhor=1;
    eD=A(depth,9)/(1000*A(depth,5));
    
    if ulum==0
    
       Flowregime='fluid only';
       A(depth, 13)=0;
       beta=0;
       Elhor=1;
    
    else
  
    if Cl<0.01 && Nfr<L1
        Flowregime='Segregated';
        A(depth, 13)=1;
        Elhor=(0.98*Cl^0.4846)/(Nfr^0.0868);
        beta=(1-Cl)*log((0.011*Nvl^3.539)/(Cl^3.768*Nfr^1.614));
        
         if beta<0
            beta=0;
        end
    
    elseif Cl>=0.01 && Nfr<L2
        Flowregime='Segregated';
        A(depth, 13)=1;
        Elhor=(0.98*Cl^0.4846)/(Nfr^0.0868);
        beta=(1-Cl)*log((0.011*Nvl^3.539)/(Cl^3.768*Nfr^1.614));
        
         if beta<0
            beta=0;
        end
       
    elseif Cl>0.01 && Nfr>L2 && Nfr<=L3
        
        Flowregime='Transition';
        A(depth, 13)=2;
        Elhorseg=(0.98*Cl^0.4846)/(Nfr^0.0868);
        Elhorint=(0.845*Cl^0.5351)/(Nfr^0.0173);
        A9=(L3-Nfr)/(L3-L2);
        B9=1-A9;
        Elhor=A9*Elhorseg+B9*Elhorint;
        beta=(1-Cl)*log((2.96*Cl^0.305*Nfr^0.0978)/(Nvl^0.4473));
        
        if beta<0
            beta=0;
        end
        
    elseif Cl>=0.01 && Cl<0.4 && Nfr>L3 && Nfr<=L1
        
        Flowregime='Intermittent';
        A(depth, 13)=3;
        Elhor=(0.845*Cl^0.5351)/(Nfr^0.0173);
        beta=(1-Cl)*log((2.96*Cl^0.305*Nfr^0.0978)/(Nvl^0.4473));
        
         if beta<0
            beta=0;
        end
        
    elseif Cl>=0.4 && Nfr>L3 && Nfr<=L4
        
        Flowregime='Intermittent';
        A(depth, 13)=3;
        Elhor=(0.845*Cl^0.5351)/(Nfr^0.0173);
        beta=(1-Cl)*log((2.96*Cl^0.305*Nfr^0.0978)/(Nvl^0.4473));
        
         if beta<0
            beta=0;      
        end
        
    elseif Cl<0.4 && Nfr>=L1
        
        Flowregime='Distrubuted';
        A(depth, 13)=4;
        Elhor=(1.065*Cl^0.5824)/(Nfr^0.0609);
        beta=0;
        
    elseif Cl>=0.4 && Nfr>L4
        
        Flowregime='Distrubuted';
        A(depth, 13)=4;
        Elhor=(1.065*Cl^0.5824)/(Nfr^0.0609);
        beta=0;
    
    end
    end
    
      Winc=90; %Wellbore inc. 
      B12=1+beta*(sin(1.8*Winc)-(1/3)*(sin(1.8*Winc))^3);
      El=Elhor*B12;
      
      if El>1  %Limitation according to Beggs and Brill 
         El=1;
      end

      if El<Cl
         El=Cl;
      end

      mixdenns=liquiddensitya*Cl+GDa*(1-Cl); %lb/ft3
      mixdenns1=liquiddensity*Cl+GD*(1-Cl); %kg/m3
      mixden=liquiddensity*El+GD*(1-El); %kg/m3
      mixden2=mixden*0.0624279606; %lb/ft3
      
%-------------------- Hydrostatic Pressure Loss --------------------%
      
      dPhyd2=((mixden2)/144); %psi/ft
      dPhyd22=((mixden2)/144)*0.226205948; %bar/m
      dPhyd2test=(mixdenns/144);
      dPhyd22test=dPhyd2test*0.226205948;
      
%------------------ Determining friction factor --------------------%
      y=(Cl/El^2);
      if y>1 && y<1.2
         S=log(2.2*y-1.2);
      else
         S=log(y)/(-0.0523+3.18*log(y)-0.872*(log(y))^2+0.01853*(log(y))^4);
      end
      
      ReNS=1488*mixdenns*V*3.28084*D/mixturevisc;
      
      if ReNS>4000
          Flow='Turbulent';
          fns=1/(-2*log10((eD/3.7065)-((5.0452/ReNS)*log10(((eD^1.1098)/2.8257)+(7.149/ReNS)^0.8981))))^2;
      elseif ReNS<2300
          Flow='Laminar';
          fdns=64/ReNS;
          fns=fdns/4;
      else 
          Flow='Transient';
          fns=1/(-2*log10((eD/3.7065)-((5.0452/ReNS)*log10(((eD^1.1098)/2.8257)+(7.149/ReNS)^0.8981))))^2;
      end
      
%------------------- Adjusting for liquid holdup -------------------%      
      
      ftp=fns*exp(S);
      fdtp=ftp*4;
      
%--------------------- Frictional Pressure Drop --------------------%
      
      dPfrictest=(2*fns*((V*3.28084)^2)*mixdenns)/(144*32.174*D);
      dPfrictestbar=dPfrictest*0.22620594;
      dPfric=(2*ftp*((V*3.28084)^2)*mixdenns)/(144*32.174*D); %psi/ft
      dPfric2=dPfric*0.22620594; %bar/m
      dPfric3=fdtp*((mixdenns*16.018463*V^2)/(2*D/3.28084))*10^-5; %bar/m
      Nextpressure=P-dPfric3-dPhyd22;
   
      if Nextpressure<SP
         Nextpressure=SP;
      end

      A(depth-1,4)=Nextpressure;
      A(depth, 6)=dPfric2;
      A(depth, 7)=dPhyd22;
      A(depth, 8)=V;
      A(depth, 9)=GD;
      A(depth, 11)=mixden;
      A(depth, 12)=liquiddensity;
      A(depth, 15)=ftp;
      A(depth, 16)=fns;

end
Constraint=A(1,4);
if Constraint>(SP+40)
    %RR=RR+10
    R=R+10
elseif Constraint<=(SP+50) && Constraint>(SP+15)
    %RR=RR+1
    R=R+1
elseif Constraint<=(SP+15) && Constraint>(SP+1)
    %RR=RR+0.1
    R=R+0.1
else
    %RR=RR+0.01
    R=R+0.01
end
end
    Fricsum=0;
    Hydsum=0;
for i=1:3000
    id=3001-i;
    Fricsum=Fricsum+A(id,6);
    Hydsum=Hydsum+A(id,7);
end


Fricsum
Hydsum
Totsum=Fricsum+Hydsum
    











